import com.amazonaws.services.glue.ChoiceOption
import com.amazonaws.services.glue.GlueContext
import com.amazonaws.services.glue.MappingSpec
import com.amazonaws.services.glue.ResolveSpec
import com.amazonaws.services.glue.errors.CallSite
import com.amazonaws.services.glue.util.GlueArgParser
import com.amazonaws.services.glue.util.Job
import com.amazonaws.services.glue.util.JsonOptions
import org.apache.spark.SparkContext
import scala.collection.JavaConverters._
import com.marklogic.hub.spark._
import org.apache.spark.sql.sources.v2.writer.DataWriter

object GlueApp {
  def main(sysArgs: Array[String]) {
    val spark: SparkContext = new SparkContext()
    val glueContext: GlueContext = new GlueContext(spark)
    // @params: [JOB_NAME]
    val args = GlueArgParser.getResolvedOptions(sysArgs, Seq("JOB_NAME").toArray)
    Job.init(args("JOB_NAME"), glueContext, args.asJava)
    // @type: DataSource
    // @args: [database = "customer-db", table_name = "city_csv", transformationContext = "city"]
    // @return: city
    // @inputs: []
    val city = glueContext.getCatalogSource(database = "customer-db", tableName = "city_csv", redshiftTmpDir = "", transformationContext = "city").getDynamicFrame()
    // @type: DataSource
    // @args: [database = "customer-db", table_name = "country_csv", transformationContext = "country"]
    // @return: country
    // @inputs: []
    val country = glueContext.getCatalogSource(database = "customer-db", tableName = "country_csv", redshiftTmpDir = "", transformationContext = "country").getDynamicFrame()
    // @type: DataSource
    // @args: [database = "customer-db", table_name = "customer_csv", transformationContext = "customer"]
    // @return: customer
    // @inputs: []
    val customer = glueContext.getCatalogSource(database = "customer-db", tableName = "customer_csv", redshiftTmpDir = "", transformationContext = "customer").getDynamicFrame()
    // @type: DataSource
    // @args: [database = "customer-db", table_name = "address_csv", transformationContext = "address"]
    // @return: address
    // @inputs: []
    val address = glueContext.getCatalogSource(database = "customer-db", tableName = "address_csv", redshiftTmpDir = "", transformationContext = "address").getDynamicFrame()
   
   println("DataSource added")

     println("Schema added")

// @type: Join
// @args: [keys1 = [<keys1>], keys2 = [<keys2>]]
// @return: <output>
// @inputs: [frame1 = <frame1>, frame2 = <frame2>]
   val cityCountry = city.join(keys1 = Seq("country_id"), keys2 = Seq("country_id"), frame2 = country, transformationContext = "cityCountry")
// @type: DropFields
// @args: [paths = [<paths>], transformation_ctx = "<transformation_ctx>"]
// @return: <output>
// @inputs: [frame = <frame>]
   val cityFinal = cityCountry.dropFields(fieldNames = Seq("country_id"), transformationContext = "cityFinal")

// @type: Join
// @args: [keys1 = [<keys1>], keys2 = [<keys2>]]
// @return: <output>
// @inputs: [frame1 = <frame1>, frame2 = <frame2>]
    val custAdd = customer.join(keys1 = Seq("address_id"), keys2 = Seq("address_id"), frame2 = address, transformationContext = "custAdd")
// @type: DropFields
// @args: [paths = [<paths>], transformation_ctx = "<transformation_ctx>"]
// @return: <output>
// @inputs: [frame = <frame>]
    val custAddFinal = custAdd.dropFields(fieldNames = Seq("address_id"), transformationContext = "custAddFinal")

// @type: Join
// @args: [keys1 = [<keys1>], keys2 = [<keys2>]]
// @return: <output>
// @inputs: [frame1 = <frame1>, frame2 = <frame2>]
    val custfinal = custAddFinal.join(keys1 = Seq("city_id"), keys2 = Seq("city_id"), frame2 = cityFinal, transformationContext = "custfinal")
// @type: DropFields
// @args: [paths = [<paths>], transformation_ctx = "<transformation_ctx>"]
// @return: <output>
// @inputs: [frame = <frame>]

//custfinal.show(3)
    val customerdata = custfinal.dropFields(fieldNames = Seq("city_id"), transformationContext = "customerdata")
      println("joined")
// @type: ApplyMapping
// @args: [mappings = <mappings>, transformation_ctx = "<transformation_ctx>"]
// @return: <output>
// @inputs: [frame = <frame>]
     val customer_entity = customerdata.applyMapping(mappings = Seq(("customer_id","bigint","customer_id","bigint"),
                                        ("first_name","string","first_name","string"),
                                        ("last_name","string","last_name","string"),
                                        ("email","string","email","string"),
                                        ("address","string","address.address1","string"),
                                        ("address2","string","address.address2","string"),
                                        ("district","string","address.district","string"),
                                        ("city","string","address.city","string"),
                                        ("postal_code","bigint","address.postal_code","bigint"),
                                        ("phone","bigint","phone","bigint"),
                                        ("country","string","address.country","string"), 
                                        ("create_date","string","create_date","timestamp")), caseSensitive = false, transformationContext = "customer_entity")

    customer_entity.show(1)
    println("joined schema")
    val schema = customer_entity.schema
    println("*************"+schema+"****************")
   // customer_entity.printSchema()
     println("job completed")
// @type: DataSink
// @args: [connection_type = "s3", connection_options = {"path":"s3://aws-denormalize-stadikon/output"}, format = "json", format_options = <format_options>, transformation_ctx = "<transformation_ctx>"]
// @return: <output>
// @inputs: [frame = <frame>]

    val dataframe = customer_entity.toDF()
    println("--------- dataframe is -----------")
    dataframe.show();

    val host = "YOUR-DHS-INSTANCE-HOST"
    val port = 8010
    val secretId = "marklogic-spark-tutorial"
    val prefix = "/customer/marklogic-spark-tutorial/"
    val moduledb = "data-hub-MODULES"
    val apiPath = "apiPath";
    
    val writer = dataframe.write.format("com.marklogic.hub.spark.sql.sources.v2.DefaultSource")
                    .option("mlHost", host)
                    .option("mlStagingPort", port)
                    .option("mlUsername", "YOUR-DHS-INSTANCE-USERNAME")
                    .option("mlPassword", "YOUR-DHS-INSTANCE-PASSWORD")
                    .option("uriprefix", prefix)
                    .option("permissions", "data-hub-operator,read,data-hub-operator,update")
                    .option("hubDhs", "true")
                    .option("hubSsl", "true");
                    
    val t2 =  writer.save;

  }
}